# frozen_string_literal: true

module PlanetExpress
  class ShipmentDiscountCalculator
    attr_reader :shipment_repository, :rules

    def initialize(shipment_repository:,
                   rules:)
      @shipment_repository = shipment_repository
      @rules = rules
    end

    def run(shipment_reader)
      shipment_reader.each do |shipment|
        shipment.if_valid do |valid_shipment|
          apply_rules(valid_shipment)
        end
        shipment_repository.add(shipment)
      end
    end

    private

    def apply_rules(valid_shipment)
      rules.each do |rule|
        rule.apply_to(valid_shipment)
      end
    end
  end
end
