# frozen_string_literal: true

module PlanetExpress
  class ShipmentFormatter
    def format(shipment)
      return format_invalid_shipment(shipment) if shipment.instance_of?(InvalidShipment)

      format_valid_shipment(shipment)
    end

    private

    def format_invalid_shipment(shipment)
      "#{shipment.raw_string} IGNORED"
    end

    def format_valid_shipment(shipment)
      date = shipment.date.strftime("%Y-%m-%d")
      package_size = shipment.shipping_option.package_size
      provider = shipment.shipping_option.provider
      price = shipment.price_with_discounts
      discount = shipment.total_discount_amount

      "#{date} #{package_size} #{provider} #{format_money(price)} #{discount.zero? ? '-' : format_money(discount)}"
    end

    def format_money(amount)
      amount.to_s.rjust(3, '0').insert(-3, '.')
    end
  end
end
