# frozen_string_literal: true

RSpec.describe PlanetExpress::SShipmentDiscountRule do
  let(:budget_repo) { PlanetExpress::DiscountBudgetForMonthRepository.new }
  let(:shipping_option_repo) { PlanetExpress::ShippingOptionRepository.new }
  let(:shipment_repo) { PlanetExpress::ShipmentRepository.new(data: []) }

  subject do
    described_class.new(
      discount_budget_for_month_repository: budget_repo,
      shipping_option_repository: shipping_option_repo,
      shipment_repository: shipment_repo
    )
  end

  context 's_shipment' do
    context 'more expensive than cheapest option' do
      let(:shipping_option) do
        shipping_option_repo.find_by_provider_and_package_size('MR', 'S')
      end

      let(:shipment) do
        PlanetExpress::Shipment.new(Date.today,
                                    shipping_option)
      end

      it 'does applies the discount' do
        expect do
          subject.apply_to(shipment)
        end.to(change { shipment.discounts.length }.by(1))

        expect(shipment.discounts.last.amount).to eq(50)
      end

      it 'changes the discount budget' do
        expect do
          subject.apply_to(shipment)
        end.to(change { budget_repo.for_month(shipment.month).available_budget }.by(-50))
      end
    end

    context 'equals cheapest options' do
      let(:shipping_option) do
        shipping_option_repo.find_by_provider_and_package_size('LP', 'S')
      end

      let(:shipment) do
        PlanetExpress::Shipment.new(Date.today,
                                    shipping_option)
      end

      it 'does not apply the discount' do
        expect do
          subject.apply_to(shipment)
        end.to_not(change { shipment.discounts })
      end
    end
  end

  context 'non s_shipment' do
    let(:shipping_option) do
      shipping_option_repo.find_by_provider_and_package_size('LP', 'M')
    end

    let(:shipment) do
      PlanetExpress::Shipment.new(Date.today,
                                  shipping_option)
    end

    it 'does not apply the discount' do
      expect do
        subject.apply_to(shipment)
      end.to_not(change { shipment.discounts })
    end
  end
end
