# frozen_string_literal: true

RSpec.describe PlanetExpress::ShipmentDiscountCalculator do
  let(:shipment_repository) do
    PlanetExpress::ShippingOptionRepository.new(data: [])
  end
  let(:rule) do
    double('Rule')
  end
  let(:rules) do
    [rule]
  end

  subject do
    described_class.new(shipment_repository: shipment_repository,
                        rules: rules)
  end

  describe '#run' do
    context 'with valid shipment' do
      let(:shipment_reader) do
        [PlanetExpress::Shipment.new(Date.today, build(:shipping_option))]
      end

      it 'does call the rule on the invalid shipment' do
        expect(rule).to receive(:apply_to)

        subject.run(shipment_reader)
      end

      it 'adds the shipment to the shipment_repository' do
        expect(rule).to receive(:apply_to)

        expect do
          subject.run(shipment_reader)
        end.to(change { shipment_repository.all.length }.by(1))
      end
    end

    context 'with invalid shipment' do
      let(:shipment_reader) do
        [PlanetExpress::InvalidShipment.new('Invalid')]
      end

      it 'does not call the rule with the invalid shipment' do
        expect(rule).to_not receive(:apply_to)

        subject.run(shipment_reader)
      end

      it 'adds the shipment to the shipment_repository' do
        expect do
          subject.run(shipment_reader)
        end.to(change { shipment_repository.all.length }.by(1))
      end
    end
  end
end
